from enum import Enum
import math
import string


class PowerDensityUnits(Enum):
        """
            PowerDensityUnits enumeration
        """
        
        WattPerCubicMeter = 'watt_per_cubic_meter'
        """
            
        """
        
        WattPerCubicInch = 'watt_per_cubic_inch'
        """
            
        """
        
        WattPerCubicFoot = 'watt_per_cubic_foot'
        """
            
        """
        
        WattPerLiter = 'watt_per_liter'
        """
            
        """
        
        PicowattPerCubicMeter = 'picowatt_per_cubic_meter'
        """
            
        """
        
        NanowattPerCubicMeter = 'nanowatt_per_cubic_meter'
        """
            
        """
        
        MicrowattPerCubicMeter = 'microwatt_per_cubic_meter'
        """
            
        """
        
        MilliwattPerCubicMeter = 'milliwatt_per_cubic_meter'
        """
            
        """
        
        DeciwattPerCubicMeter = 'deciwatt_per_cubic_meter'
        """
            
        """
        
        DecawattPerCubicMeter = 'decawatt_per_cubic_meter'
        """
            
        """
        
        KilowattPerCubicMeter = 'kilowatt_per_cubic_meter'
        """
            
        """
        
        MegawattPerCubicMeter = 'megawatt_per_cubic_meter'
        """
            
        """
        
        GigawattPerCubicMeter = 'gigawatt_per_cubic_meter'
        """
            
        """
        
        TerawattPerCubicMeter = 'terawatt_per_cubic_meter'
        """
            
        """
        
        PicowattPerCubicInch = 'picowatt_per_cubic_inch'
        """
            
        """
        
        NanowattPerCubicInch = 'nanowatt_per_cubic_inch'
        """
            
        """
        
        MicrowattPerCubicInch = 'microwatt_per_cubic_inch'
        """
            
        """
        
        MilliwattPerCubicInch = 'milliwatt_per_cubic_inch'
        """
            
        """
        
        DeciwattPerCubicInch = 'deciwatt_per_cubic_inch'
        """
            
        """
        
        DecawattPerCubicInch = 'decawatt_per_cubic_inch'
        """
            
        """
        
        KilowattPerCubicInch = 'kilowatt_per_cubic_inch'
        """
            
        """
        
        MegawattPerCubicInch = 'megawatt_per_cubic_inch'
        """
            
        """
        
        GigawattPerCubicInch = 'gigawatt_per_cubic_inch'
        """
            
        """
        
        TerawattPerCubicInch = 'terawatt_per_cubic_inch'
        """
            
        """
        
        PicowattPerCubicFoot = 'picowatt_per_cubic_foot'
        """
            
        """
        
        NanowattPerCubicFoot = 'nanowatt_per_cubic_foot'
        """
            
        """
        
        MicrowattPerCubicFoot = 'microwatt_per_cubic_foot'
        """
            
        """
        
        MilliwattPerCubicFoot = 'milliwatt_per_cubic_foot'
        """
            
        """
        
        DeciwattPerCubicFoot = 'deciwatt_per_cubic_foot'
        """
            
        """
        
        DecawattPerCubicFoot = 'decawatt_per_cubic_foot'
        """
            
        """
        
        KilowattPerCubicFoot = 'kilowatt_per_cubic_foot'
        """
            
        """
        
        MegawattPerCubicFoot = 'megawatt_per_cubic_foot'
        """
            
        """
        
        GigawattPerCubicFoot = 'gigawatt_per_cubic_foot'
        """
            
        """
        
        TerawattPerCubicFoot = 'terawatt_per_cubic_foot'
        """
            
        """
        
        PicowattPerLiter = 'picowatt_per_liter'
        """
            
        """
        
        NanowattPerLiter = 'nanowatt_per_liter'
        """
            
        """
        
        MicrowattPerLiter = 'microwatt_per_liter'
        """
            
        """
        
        MilliwattPerLiter = 'milliwatt_per_liter'
        """
            
        """
        
        DeciwattPerLiter = 'deciwatt_per_liter'
        """
            
        """
        
        DecawattPerLiter = 'decawatt_per_liter'
        """
            
        """
        
        KilowattPerLiter = 'kilowatt_per_liter'
        """
            
        """
        
        MegawattPerLiter = 'megawatt_per_liter'
        """
            
        """
        
        GigawattPerLiter = 'gigawatt_per_liter'
        """
            
        """
        
        TerawattPerLiter = 'terawatt_per_liter'
        """
            
        """
        

class PowerDensity:
    """
    The amount of power in a volume.

    Args:
        value (float): The value.
        from_unit (PowerDensityUnits): The PowerDensity unit to create from, The default unit is WattPerCubicMeter
    """
    def __init__(self, value: float, from_unit: PowerDensityUnits = PowerDensityUnits.WattPerCubicMeter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__watts_per_cubic_meter = None
        
        self.__watts_per_cubic_inch = None
        
        self.__watts_per_cubic_foot = None
        
        self.__watts_per_liter = None
        
        self.__picowatts_per_cubic_meter = None
        
        self.__nanowatts_per_cubic_meter = None
        
        self.__microwatts_per_cubic_meter = None
        
        self.__milliwatts_per_cubic_meter = None
        
        self.__deciwatts_per_cubic_meter = None
        
        self.__decawatts_per_cubic_meter = None
        
        self.__kilowatts_per_cubic_meter = None
        
        self.__megawatts_per_cubic_meter = None
        
        self.__gigawatts_per_cubic_meter = None
        
        self.__terawatts_per_cubic_meter = None
        
        self.__picowatts_per_cubic_inch = None
        
        self.__nanowatts_per_cubic_inch = None
        
        self.__microwatts_per_cubic_inch = None
        
        self.__milliwatts_per_cubic_inch = None
        
        self.__deciwatts_per_cubic_inch = None
        
        self.__decawatts_per_cubic_inch = None
        
        self.__kilowatts_per_cubic_inch = None
        
        self.__megawatts_per_cubic_inch = None
        
        self.__gigawatts_per_cubic_inch = None
        
        self.__terawatts_per_cubic_inch = None
        
        self.__picowatts_per_cubic_foot = None
        
        self.__nanowatts_per_cubic_foot = None
        
        self.__microwatts_per_cubic_foot = None
        
        self.__milliwatts_per_cubic_foot = None
        
        self.__deciwatts_per_cubic_foot = None
        
        self.__decawatts_per_cubic_foot = None
        
        self.__kilowatts_per_cubic_foot = None
        
        self.__megawatts_per_cubic_foot = None
        
        self.__gigawatts_per_cubic_foot = None
        
        self.__terawatts_per_cubic_foot = None
        
        self.__picowatts_per_liter = None
        
        self.__nanowatts_per_liter = None
        
        self.__microwatts_per_liter = None
        
        self.__milliwatts_per_liter = None
        
        self.__deciwatts_per_liter = None
        
        self.__decawatts_per_liter = None
        
        self.__kilowatts_per_liter = None
        
        self.__megawatts_per_liter = None
        
        self.__gigawatts_per_liter = None
        
        self.__terawatts_per_liter = None
        

    def __convert_from_base(self, from_unit: PowerDensityUnits) -> float:
        value = self.__value
        
        if from_unit == PowerDensityUnits.WattPerCubicMeter:
            return (value)
        
        if from_unit == PowerDensityUnits.WattPerCubicInch:
            return (value / 6.102374409473228e4)
        
        if from_unit == PowerDensityUnits.WattPerCubicFoot:
            return (value / 3.531466672148859e1)
        
        if from_unit == PowerDensityUnits.WattPerLiter:
            return (value / 1.0e3)
        
        if from_unit == PowerDensityUnits.PicowattPerCubicMeter:
            return ((value) / 1e-12)
        
        if from_unit == PowerDensityUnits.NanowattPerCubicMeter:
            return ((value) / 1e-09)
        
        if from_unit == PowerDensityUnits.MicrowattPerCubicMeter:
            return ((value) / 1e-06)
        
        if from_unit == PowerDensityUnits.MilliwattPerCubicMeter:
            return ((value) / 0.001)
        
        if from_unit == PowerDensityUnits.DeciwattPerCubicMeter:
            return ((value) / 0.1)
        
        if from_unit == PowerDensityUnits.DecawattPerCubicMeter:
            return ((value) / 10.0)
        
        if from_unit == PowerDensityUnits.KilowattPerCubicMeter:
            return ((value) / 1000.0)
        
        if from_unit == PowerDensityUnits.MegawattPerCubicMeter:
            return ((value) / 1000000.0)
        
        if from_unit == PowerDensityUnits.GigawattPerCubicMeter:
            return ((value) / 1000000000.0)
        
        if from_unit == PowerDensityUnits.TerawattPerCubicMeter:
            return ((value) / 1000000000000.0)
        
        if from_unit == PowerDensityUnits.PicowattPerCubicInch:
            return ((value / 6.102374409473228e4) / 1e-12)
        
        if from_unit == PowerDensityUnits.NanowattPerCubicInch:
            return ((value / 6.102374409473228e4) / 1e-09)
        
        if from_unit == PowerDensityUnits.MicrowattPerCubicInch:
            return ((value / 6.102374409473228e4) / 1e-06)
        
        if from_unit == PowerDensityUnits.MilliwattPerCubicInch:
            return ((value / 6.102374409473228e4) / 0.001)
        
        if from_unit == PowerDensityUnits.DeciwattPerCubicInch:
            return ((value / 6.102374409473228e4) / 0.1)
        
        if from_unit == PowerDensityUnits.DecawattPerCubicInch:
            return ((value / 6.102374409473228e4) / 10.0)
        
        if from_unit == PowerDensityUnits.KilowattPerCubicInch:
            return ((value / 6.102374409473228e4) / 1000.0)
        
        if from_unit == PowerDensityUnits.MegawattPerCubicInch:
            return ((value / 6.102374409473228e4) / 1000000.0)
        
        if from_unit == PowerDensityUnits.GigawattPerCubicInch:
            return ((value / 6.102374409473228e4) / 1000000000.0)
        
        if from_unit == PowerDensityUnits.TerawattPerCubicInch:
            return ((value / 6.102374409473228e4) / 1000000000000.0)
        
        if from_unit == PowerDensityUnits.PicowattPerCubicFoot:
            return ((value / 3.531466672148859e1) / 1e-12)
        
        if from_unit == PowerDensityUnits.NanowattPerCubicFoot:
            return ((value / 3.531466672148859e1) / 1e-09)
        
        if from_unit == PowerDensityUnits.MicrowattPerCubicFoot:
            return ((value / 3.531466672148859e1) / 1e-06)
        
        if from_unit == PowerDensityUnits.MilliwattPerCubicFoot:
            return ((value / 3.531466672148859e1) / 0.001)
        
        if from_unit == PowerDensityUnits.DeciwattPerCubicFoot:
            return ((value / 3.531466672148859e1) / 0.1)
        
        if from_unit == PowerDensityUnits.DecawattPerCubicFoot:
            return ((value / 3.531466672148859e1) / 10.0)
        
        if from_unit == PowerDensityUnits.KilowattPerCubicFoot:
            return ((value / 3.531466672148859e1) / 1000.0)
        
        if from_unit == PowerDensityUnits.MegawattPerCubicFoot:
            return ((value / 3.531466672148859e1) / 1000000.0)
        
        if from_unit == PowerDensityUnits.GigawattPerCubicFoot:
            return ((value / 3.531466672148859e1) / 1000000000.0)
        
        if from_unit == PowerDensityUnits.TerawattPerCubicFoot:
            return ((value / 3.531466672148859e1) / 1000000000000.0)
        
        if from_unit == PowerDensityUnits.PicowattPerLiter:
            return ((value / 1.0e3) / 1e-12)
        
        if from_unit == PowerDensityUnits.NanowattPerLiter:
            return ((value / 1.0e3) / 1e-09)
        
        if from_unit == PowerDensityUnits.MicrowattPerLiter:
            return ((value / 1.0e3) / 1e-06)
        
        if from_unit == PowerDensityUnits.MilliwattPerLiter:
            return ((value / 1.0e3) / 0.001)
        
        if from_unit == PowerDensityUnits.DeciwattPerLiter:
            return ((value / 1.0e3) / 0.1)
        
        if from_unit == PowerDensityUnits.DecawattPerLiter:
            return ((value / 1.0e3) / 10.0)
        
        if from_unit == PowerDensityUnits.KilowattPerLiter:
            return ((value / 1.0e3) / 1000.0)
        
        if from_unit == PowerDensityUnits.MegawattPerLiter:
            return ((value / 1.0e3) / 1000000.0)
        
        if from_unit == PowerDensityUnits.GigawattPerLiter:
            return ((value / 1.0e3) / 1000000000.0)
        
        if from_unit == PowerDensityUnits.TerawattPerLiter:
            return ((value / 1.0e3) / 1000000000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: PowerDensityUnits) -> float:
        
        if to_unit == PowerDensityUnits.WattPerCubicMeter:
            return (value)
        
        if to_unit == PowerDensityUnits.WattPerCubicInch:
            return (value * 6.102374409473228e4)
        
        if to_unit == PowerDensityUnits.WattPerCubicFoot:
            return (value * 3.531466672148859e1)
        
        if to_unit == PowerDensityUnits.WattPerLiter:
            return (value * 1.0e3)
        
        if to_unit == PowerDensityUnits.PicowattPerCubicMeter:
            return ((value) * 1e-12)
        
        if to_unit == PowerDensityUnits.NanowattPerCubicMeter:
            return ((value) * 1e-09)
        
        if to_unit == PowerDensityUnits.MicrowattPerCubicMeter:
            return ((value) * 1e-06)
        
        if to_unit == PowerDensityUnits.MilliwattPerCubicMeter:
            return ((value) * 0.001)
        
        if to_unit == PowerDensityUnits.DeciwattPerCubicMeter:
            return ((value) * 0.1)
        
        if to_unit == PowerDensityUnits.DecawattPerCubicMeter:
            return ((value) * 10.0)
        
        if to_unit == PowerDensityUnits.KilowattPerCubicMeter:
            return ((value) * 1000.0)
        
        if to_unit == PowerDensityUnits.MegawattPerCubicMeter:
            return ((value) * 1000000.0)
        
        if to_unit == PowerDensityUnits.GigawattPerCubicMeter:
            return ((value) * 1000000000.0)
        
        if to_unit == PowerDensityUnits.TerawattPerCubicMeter:
            return ((value) * 1000000000000.0)
        
        if to_unit == PowerDensityUnits.PicowattPerCubicInch:
            return ((value * 6.102374409473228e4) * 1e-12)
        
        if to_unit == PowerDensityUnits.NanowattPerCubicInch:
            return ((value * 6.102374409473228e4) * 1e-09)
        
        if to_unit == PowerDensityUnits.MicrowattPerCubicInch:
            return ((value * 6.102374409473228e4) * 1e-06)
        
        if to_unit == PowerDensityUnits.MilliwattPerCubicInch:
            return ((value * 6.102374409473228e4) * 0.001)
        
        if to_unit == PowerDensityUnits.DeciwattPerCubicInch:
            return ((value * 6.102374409473228e4) * 0.1)
        
        if to_unit == PowerDensityUnits.DecawattPerCubicInch:
            return ((value * 6.102374409473228e4) * 10.0)
        
        if to_unit == PowerDensityUnits.KilowattPerCubicInch:
            return ((value * 6.102374409473228e4) * 1000.0)
        
        if to_unit == PowerDensityUnits.MegawattPerCubicInch:
            return ((value * 6.102374409473228e4) * 1000000.0)
        
        if to_unit == PowerDensityUnits.GigawattPerCubicInch:
            return ((value * 6.102374409473228e4) * 1000000000.0)
        
        if to_unit == PowerDensityUnits.TerawattPerCubicInch:
            return ((value * 6.102374409473228e4) * 1000000000000.0)
        
        if to_unit == PowerDensityUnits.PicowattPerCubicFoot:
            return ((value * 3.531466672148859e1) * 1e-12)
        
        if to_unit == PowerDensityUnits.NanowattPerCubicFoot:
            return ((value * 3.531466672148859e1) * 1e-09)
        
        if to_unit == PowerDensityUnits.MicrowattPerCubicFoot:
            return ((value * 3.531466672148859e1) * 1e-06)
        
        if to_unit == PowerDensityUnits.MilliwattPerCubicFoot:
            return ((value * 3.531466672148859e1) * 0.001)
        
        if to_unit == PowerDensityUnits.DeciwattPerCubicFoot:
            return ((value * 3.531466672148859e1) * 0.1)
        
        if to_unit == PowerDensityUnits.DecawattPerCubicFoot:
            return ((value * 3.531466672148859e1) * 10.0)
        
        if to_unit == PowerDensityUnits.KilowattPerCubicFoot:
            return ((value * 3.531466672148859e1) * 1000.0)
        
        if to_unit == PowerDensityUnits.MegawattPerCubicFoot:
            return ((value * 3.531466672148859e1) * 1000000.0)
        
        if to_unit == PowerDensityUnits.GigawattPerCubicFoot:
            return ((value * 3.531466672148859e1) * 1000000000.0)
        
        if to_unit == PowerDensityUnits.TerawattPerCubicFoot:
            return ((value * 3.531466672148859e1) * 1000000000000.0)
        
        if to_unit == PowerDensityUnits.PicowattPerLiter:
            return ((value * 1.0e3) * 1e-12)
        
        if to_unit == PowerDensityUnits.NanowattPerLiter:
            return ((value * 1.0e3) * 1e-09)
        
        if to_unit == PowerDensityUnits.MicrowattPerLiter:
            return ((value * 1.0e3) * 1e-06)
        
        if to_unit == PowerDensityUnits.MilliwattPerLiter:
            return ((value * 1.0e3) * 0.001)
        
        if to_unit == PowerDensityUnits.DeciwattPerLiter:
            return ((value * 1.0e3) * 0.1)
        
        if to_unit == PowerDensityUnits.DecawattPerLiter:
            return ((value * 1.0e3) * 10.0)
        
        if to_unit == PowerDensityUnits.KilowattPerLiter:
            return ((value * 1.0e3) * 1000.0)
        
        if to_unit == PowerDensityUnits.MegawattPerLiter:
            return ((value * 1.0e3) * 1000000.0)
        
        if to_unit == PowerDensityUnits.GigawattPerLiter:
            return ((value * 1.0e3) * 1000000000.0)
        
        if to_unit == PowerDensityUnits.TerawattPerLiter:
            return ((value * 1.0e3) * 1000000000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_watts_per_cubic_meter(watts_per_cubic_meter: float):
        """
        Create a new instance of PowerDensity from a value in watts_per_cubic_meter.

        

        :param meters: The PowerDensity value in watts_per_cubic_meter.
        :type watts_per_cubic_meter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(watts_per_cubic_meter, PowerDensityUnits.WattPerCubicMeter)

    
    @staticmethod
    def from_watts_per_cubic_inch(watts_per_cubic_inch: float):
        """
        Create a new instance of PowerDensity from a value in watts_per_cubic_inch.

        

        :param meters: The PowerDensity value in watts_per_cubic_inch.
        :type watts_per_cubic_inch: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(watts_per_cubic_inch, PowerDensityUnits.WattPerCubicInch)

    
    @staticmethod
    def from_watts_per_cubic_foot(watts_per_cubic_foot: float):
        """
        Create a new instance of PowerDensity from a value in watts_per_cubic_foot.

        

        :param meters: The PowerDensity value in watts_per_cubic_foot.
        :type watts_per_cubic_foot: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(watts_per_cubic_foot, PowerDensityUnits.WattPerCubicFoot)

    
    @staticmethod
    def from_watts_per_liter(watts_per_liter: float):
        """
        Create a new instance of PowerDensity from a value in watts_per_liter.

        

        :param meters: The PowerDensity value in watts_per_liter.
        :type watts_per_liter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(watts_per_liter, PowerDensityUnits.WattPerLiter)

    
    @staticmethod
    def from_picowatts_per_cubic_meter(picowatts_per_cubic_meter: float):
        """
        Create a new instance of PowerDensity from a value in picowatts_per_cubic_meter.

        

        :param meters: The PowerDensity value in picowatts_per_cubic_meter.
        :type picowatts_per_cubic_meter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(picowatts_per_cubic_meter, PowerDensityUnits.PicowattPerCubicMeter)

    
    @staticmethod
    def from_nanowatts_per_cubic_meter(nanowatts_per_cubic_meter: float):
        """
        Create a new instance of PowerDensity from a value in nanowatts_per_cubic_meter.

        

        :param meters: The PowerDensity value in nanowatts_per_cubic_meter.
        :type nanowatts_per_cubic_meter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(nanowatts_per_cubic_meter, PowerDensityUnits.NanowattPerCubicMeter)

    
    @staticmethod
    def from_microwatts_per_cubic_meter(microwatts_per_cubic_meter: float):
        """
        Create a new instance of PowerDensity from a value in microwatts_per_cubic_meter.

        

        :param meters: The PowerDensity value in microwatts_per_cubic_meter.
        :type microwatts_per_cubic_meter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(microwatts_per_cubic_meter, PowerDensityUnits.MicrowattPerCubicMeter)

    
    @staticmethod
    def from_milliwatts_per_cubic_meter(milliwatts_per_cubic_meter: float):
        """
        Create a new instance of PowerDensity from a value in milliwatts_per_cubic_meter.

        

        :param meters: The PowerDensity value in milliwatts_per_cubic_meter.
        :type milliwatts_per_cubic_meter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(milliwatts_per_cubic_meter, PowerDensityUnits.MilliwattPerCubicMeter)

    
    @staticmethod
    def from_deciwatts_per_cubic_meter(deciwatts_per_cubic_meter: float):
        """
        Create a new instance of PowerDensity from a value in deciwatts_per_cubic_meter.

        

        :param meters: The PowerDensity value in deciwatts_per_cubic_meter.
        :type deciwatts_per_cubic_meter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(deciwatts_per_cubic_meter, PowerDensityUnits.DeciwattPerCubicMeter)

    
    @staticmethod
    def from_decawatts_per_cubic_meter(decawatts_per_cubic_meter: float):
        """
        Create a new instance of PowerDensity from a value in decawatts_per_cubic_meter.

        

        :param meters: The PowerDensity value in decawatts_per_cubic_meter.
        :type decawatts_per_cubic_meter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(decawatts_per_cubic_meter, PowerDensityUnits.DecawattPerCubicMeter)

    
    @staticmethod
    def from_kilowatts_per_cubic_meter(kilowatts_per_cubic_meter: float):
        """
        Create a new instance of PowerDensity from a value in kilowatts_per_cubic_meter.

        

        :param meters: The PowerDensity value in kilowatts_per_cubic_meter.
        :type kilowatts_per_cubic_meter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(kilowatts_per_cubic_meter, PowerDensityUnits.KilowattPerCubicMeter)

    
    @staticmethod
    def from_megawatts_per_cubic_meter(megawatts_per_cubic_meter: float):
        """
        Create a new instance of PowerDensity from a value in megawatts_per_cubic_meter.

        

        :param meters: The PowerDensity value in megawatts_per_cubic_meter.
        :type megawatts_per_cubic_meter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(megawatts_per_cubic_meter, PowerDensityUnits.MegawattPerCubicMeter)

    
    @staticmethod
    def from_gigawatts_per_cubic_meter(gigawatts_per_cubic_meter: float):
        """
        Create a new instance of PowerDensity from a value in gigawatts_per_cubic_meter.

        

        :param meters: The PowerDensity value in gigawatts_per_cubic_meter.
        :type gigawatts_per_cubic_meter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(gigawatts_per_cubic_meter, PowerDensityUnits.GigawattPerCubicMeter)

    
    @staticmethod
    def from_terawatts_per_cubic_meter(terawatts_per_cubic_meter: float):
        """
        Create a new instance of PowerDensity from a value in terawatts_per_cubic_meter.

        

        :param meters: The PowerDensity value in terawatts_per_cubic_meter.
        :type terawatts_per_cubic_meter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(terawatts_per_cubic_meter, PowerDensityUnits.TerawattPerCubicMeter)

    
    @staticmethod
    def from_picowatts_per_cubic_inch(picowatts_per_cubic_inch: float):
        """
        Create a new instance of PowerDensity from a value in picowatts_per_cubic_inch.

        

        :param meters: The PowerDensity value in picowatts_per_cubic_inch.
        :type picowatts_per_cubic_inch: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(picowatts_per_cubic_inch, PowerDensityUnits.PicowattPerCubicInch)

    
    @staticmethod
    def from_nanowatts_per_cubic_inch(nanowatts_per_cubic_inch: float):
        """
        Create a new instance of PowerDensity from a value in nanowatts_per_cubic_inch.

        

        :param meters: The PowerDensity value in nanowatts_per_cubic_inch.
        :type nanowatts_per_cubic_inch: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(nanowatts_per_cubic_inch, PowerDensityUnits.NanowattPerCubicInch)

    
    @staticmethod
    def from_microwatts_per_cubic_inch(microwatts_per_cubic_inch: float):
        """
        Create a new instance of PowerDensity from a value in microwatts_per_cubic_inch.

        

        :param meters: The PowerDensity value in microwatts_per_cubic_inch.
        :type microwatts_per_cubic_inch: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(microwatts_per_cubic_inch, PowerDensityUnits.MicrowattPerCubicInch)

    
    @staticmethod
    def from_milliwatts_per_cubic_inch(milliwatts_per_cubic_inch: float):
        """
        Create a new instance of PowerDensity from a value in milliwatts_per_cubic_inch.

        

        :param meters: The PowerDensity value in milliwatts_per_cubic_inch.
        :type milliwatts_per_cubic_inch: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(milliwatts_per_cubic_inch, PowerDensityUnits.MilliwattPerCubicInch)

    
    @staticmethod
    def from_deciwatts_per_cubic_inch(deciwatts_per_cubic_inch: float):
        """
        Create a new instance of PowerDensity from a value in deciwatts_per_cubic_inch.

        

        :param meters: The PowerDensity value in deciwatts_per_cubic_inch.
        :type deciwatts_per_cubic_inch: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(deciwatts_per_cubic_inch, PowerDensityUnits.DeciwattPerCubicInch)

    
    @staticmethod
    def from_decawatts_per_cubic_inch(decawatts_per_cubic_inch: float):
        """
        Create a new instance of PowerDensity from a value in decawatts_per_cubic_inch.

        

        :param meters: The PowerDensity value in decawatts_per_cubic_inch.
        :type decawatts_per_cubic_inch: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(decawatts_per_cubic_inch, PowerDensityUnits.DecawattPerCubicInch)

    
    @staticmethod
    def from_kilowatts_per_cubic_inch(kilowatts_per_cubic_inch: float):
        """
        Create a new instance of PowerDensity from a value in kilowatts_per_cubic_inch.

        

        :param meters: The PowerDensity value in kilowatts_per_cubic_inch.
        :type kilowatts_per_cubic_inch: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(kilowatts_per_cubic_inch, PowerDensityUnits.KilowattPerCubicInch)

    
    @staticmethod
    def from_megawatts_per_cubic_inch(megawatts_per_cubic_inch: float):
        """
        Create a new instance of PowerDensity from a value in megawatts_per_cubic_inch.

        

        :param meters: The PowerDensity value in megawatts_per_cubic_inch.
        :type megawatts_per_cubic_inch: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(megawatts_per_cubic_inch, PowerDensityUnits.MegawattPerCubicInch)

    
    @staticmethod
    def from_gigawatts_per_cubic_inch(gigawatts_per_cubic_inch: float):
        """
        Create a new instance of PowerDensity from a value in gigawatts_per_cubic_inch.

        

        :param meters: The PowerDensity value in gigawatts_per_cubic_inch.
        :type gigawatts_per_cubic_inch: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(gigawatts_per_cubic_inch, PowerDensityUnits.GigawattPerCubicInch)

    
    @staticmethod
    def from_terawatts_per_cubic_inch(terawatts_per_cubic_inch: float):
        """
        Create a new instance of PowerDensity from a value in terawatts_per_cubic_inch.

        

        :param meters: The PowerDensity value in terawatts_per_cubic_inch.
        :type terawatts_per_cubic_inch: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(terawatts_per_cubic_inch, PowerDensityUnits.TerawattPerCubicInch)

    
    @staticmethod
    def from_picowatts_per_cubic_foot(picowatts_per_cubic_foot: float):
        """
        Create a new instance of PowerDensity from a value in picowatts_per_cubic_foot.

        

        :param meters: The PowerDensity value in picowatts_per_cubic_foot.
        :type picowatts_per_cubic_foot: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(picowatts_per_cubic_foot, PowerDensityUnits.PicowattPerCubicFoot)

    
    @staticmethod
    def from_nanowatts_per_cubic_foot(nanowatts_per_cubic_foot: float):
        """
        Create a new instance of PowerDensity from a value in nanowatts_per_cubic_foot.

        

        :param meters: The PowerDensity value in nanowatts_per_cubic_foot.
        :type nanowatts_per_cubic_foot: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(nanowatts_per_cubic_foot, PowerDensityUnits.NanowattPerCubicFoot)

    
    @staticmethod
    def from_microwatts_per_cubic_foot(microwatts_per_cubic_foot: float):
        """
        Create a new instance of PowerDensity from a value in microwatts_per_cubic_foot.

        

        :param meters: The PowerDensity value in microwatts_per_cubic_foot.
        :type microwatts_per_cubic_foot: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(microwatts_per_cubic_foot, PowerDensityUnits.MicrowattPerCubicFoot)

    
    @staticmethod
    def from_milliwatts_per_cubic_foot(milliwatts_per_cubic_foot: float):
        """
        Create a new instance of PowerDensity from a value in milliwatts_per_cubic_foot.

        

        :param meters: The PowerDensity value in milliwatts_per_cubic_foot.
        :type milliwatts_per_cubic_foot: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(milliwatts_per_cubic_foot, PowerDensityUnits.MilliwattPerCubicFoot)

    
    @staticmethod
    def from_deciwatts_per_cubic_foot(deciwatts_per_cubic_foot: float):
        """
        Create a new instance of PowerDensity from a value in deciwatts_per_cubic_foot.

        

        :param meters: The PowerDensity value in deciwatts_per_cubic_foot.
        :type deciwatts_per_cubic_foot: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(deciwatts_per_cubic_foot, PowerDensityUnits.DeciwattPerCubicFoot)

    
    @staticmethod
    def from_decawatts_per_cubic_foot(decawatts_per_cubic_foot: float):
        """
        Create a new instance of PowerDensity from a value in decawatts_per_cubic_foot.

        

        :param meters: The PowerDensity value in decawatts_per_cubic_foot.
        :type decawatts_per_cubic_foot: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(decawatts_per_cubic_foot, PowerDensityUnits.DecawattPerCubicFoot)

    
    @staticmethod
    def from_kilowatts_per_cubic_foot(kilowatts_per_cubic_foot: float):
        """
        Create a new instance of PowerDensity from a value in kilowatts_per_cubic_foot.

        

        :param meters: The PowerDensity value in kilowatts_per_cubic_foot.
        :type kilowatts_per_cubic_foot: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(kilowatts_per_cubic_foot, PowerDensityUnits.KilowattPerCubicFoot)

    
    @staticmethod
    def from_megawatts_per_cubic_foot(megawatts_per_cubic_foot: float):
        """
        Create a new instance of PowerDensity from a value in megawatts_per_cubic_foot.

        

        :param meters: The PowerDensity value in megawatts_per_cubic_foot.
        :type megawatts_per_cubic_foot: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(megawatts_per_cubic_foot, PowerDensityUnits.MegawattPerCubicFoot)

    
    @staticmethod
    def from_gigawatts_per_cubic_foot(gigawatts_per_cubic_foot: float):
        """
        Create a new instance of PowerDensity from a value in gigawatts_per_cubic_foot.

        

        :param meters: The PowerDensity value in gigawatts_per_cubic_foot.
        :type gigawatts_per_cubic_foot: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(gigawatts_per_cubic_foot, PowerDensityUnits.GigawattPerCubicFoot)

    
    @staticmethod
    def from_terawatts_per_cubic_foot(terawatts_per_cubic_foot: float):
        """
        Create a new instance of PowerDensity from a value in terawatts_per_cubic_foot.

        

        :param meters: The PowerDensity value in terawatts_per_cubic_foot.
        :type terawatts_per_cubic_foot: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(terawatts_per_cubic_foot, PowerDensityUnits.TerawattPerCubicFoot)

    
    @staticmethod
    def from_picowatts_per_liter(picowatts_per_liter: float):
        """
        Create a new instance of PowerDensity from a value in picowatts_per_liter.

        

        :param meters: The PowerDensity value in picowatts_per_liter.
        :type picowatts_per_liter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(picowatts_per_liter, PowerDensityUnits.PicowattPerLiter)

    
    @staticmethod
    def from_nanowatts_per_liter(nanowatts_per_liter: float):
        """
        Create a new instance of PowerDensity from a value in nanowatts_per_liter.

        

        :param meters: The PowerDensity value in nanowatts_per_liter.
        :type nanowatts_per_liter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(nanowatts_per_liter, PowerDensityUnits.NanowattPerLiter)

    
    @staticmethod
    def from_microwatts_per_liter(microwatts_per_liter: float):
        """
        Create a new instance of PowerDensity from a value in microwatts_per_liter.

        

        :param meters: The PowerDensity value in microwatts_per_liter.
        :type microwatts_per_liter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(microwatts_per_liter, PowerDensityUnits.MicrowattPerLiter)

    
    @staticmethod
    def from_milliwatts_per_liter(milliwatts_per_liter: float):
        """
        Create a new instance of PowerDensity from a value in milliwatts_per_liter.

        

        :param meters: The PowerDensity value in milliwatts_per_liter.
        :type milliwatts_per_liter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(milliwatts_per_liter, PowerDensityUnits.MilliwattPerLiter)

    
    @staticmethod
    def from_deciwatts_per_liter(deciwatts_per_liter: float):
        """
        Create a new instance of PowerDensity from a value in deciwatts_per_liter.

        

        :param meters: The PowerDensity value in deciwatts_per_liter.
        :type deciwatts_per_liter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(deciwatts_per_liter, PowerDensityUnits.DeciwattPerLiter)

    
    @staticmethod
    def from_decawatts_per_liter(decawatts_per_liter: float):
        """
        Create a new instance of PowerDensity from a value in decawatts_per_liter.

        

        :param meters: The PowerDensity value in decawatts_per_liter.
        :type decawatts_per_liter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(decawatts_per_liter, PowerDensityUnits.DecawattPerLiter)

    
    @staticmethod
    def from_kilowatts_per_liter(kilowatts_per_liter: float):
        """
        Create a new instance of PowerDensity from a value in kilowatts_per_liter.

        

        :param meters: The PowerDensity value in kilowatts_per_liter.
        :type kilowatts_per_liter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(kilowatts_per_liter, PowerDensityUnits.KilowattPerLiter)

    
    @staticmethod
    def from_megawatts_per_liter(megawatts_per_liter: float):
        """
        Create a new instance of PowerDensity from a value in megawatts_per_liter.

        

        :param meters: The PowerDensity value in megawatts_per_liter.
        :type megawatts_per_liter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(megawatts_per_liter, PowerDensityUnits.MegawattPerLiter)

    
    @staticmethod
    def from_gigawatts_per_liter(gigawatts_per_liter: float):
        """
        Create a new instance of PowerDensity from a value in gigawatts_per_liter.

        

        :param meters: The PowerDensity value in gigawatts_per_liter.
        :type gigawatts_per_liter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(gigawatts_per_liter, PowerDensityUnits.GigawattPerLiter)

    
    @staticmethod
    def from_terawatts_per_liter(terawatts_per_liter: float):
        """
        Create a new instance of PowerDensity from a value in terawatts_per_liter.

        

        :param meters: The PowerDensity value in terawatts_per_liter.
        :type terawatts_per_liter: float
        :return: A new instance of PowerDensity.
        :rtype: PowerDensity
        """
        return PowerDensity(terawatts_per_liter, PowerDensityUnits.TerawattPerLiter)

    
    @property
    def watts_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__watts_per_cubic_meter != None:
            return self.__watts_per_cubic_meter
        self.__watts_per_cubic_meter = self.__convert_from_base(PowerDensityUnits.WattPerCubicMeter)
        return self.__watts_per_cubic_meter

    
    @property
    def watts_per_cubic_inch(self) -> float:
        """
        
        """
        if self.__watts_per_cubic_inch != None:
            return self.__watts_per_cubic_inch
        self.__watts_per_cubic_inch = self.__convert_from_base(PowerDensityUnits.WattPerCubicInch)
        return self.__watts_per_cubic_inch

    
    @property
    def watts_per_cubic_foot(self) -> float:
        """
        
        """
        if self.__watts_per_cubic_foot != None:
            return self.__watts_per_cubic_foot
        self.__watts_per_cubic_foot = self.__convert_from_base(PowerDensityUnits.WattPerCubicFoot)
        return self.__watts_per_cubic_foot

    
    @property
    def watts_per_liter(self) -> float:
        """
        
        """
        if self.__watts_per_liter != None:
            return self.__watts_per_liter
        self.__watts_per_liter = self.__convert_from_base(PowerDensityUnits.WattPerLiter)
        return self.__watts_per_liter

    
    @property
    def picowatts_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__picowatts_per_cubic_meter != None:
            return self.__picowatts_per_cubic_meter
        self.__picowatts_per_cubic_meter = self.__convert_from_base(PowerDensityUnits.PicowattPerCubicMeter)
        return self.__picowatts_per_cubic_meter

    
    @property
    def nanowatts_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__nanowatts_per_cubic_meter != None:
            return self.__nanowatts_per_cubic_meter
        self.__nanowatts_per_cubic_meter = self.__convert_from_base(PowerDensityUnits.NanowattPerCubicMeter)
        return self.__nanowatts_per_cubic_meter

    
    @property
    def microwatts_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__microwatts_per_cubic_meter != None:
            return self.__microwatts_per_cubic_meter
        self.__microwatts_per_cubic_meter = self.__convert_from_base(PowerDensityUnits.MicrowattPerCubicMeter)
        return self.__microwatts_per_cubic_meter

    
    @property
    def milliwatts_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__milliwatts_per_cubic_meter != None:
            return self.__milliwatts_per_cubic_meter
        self.__milliwatts_per_cubic_meter = self.__convert_from_base(PowerDensityUnits.MilliwattPerCubicMeter)
        return self.__milliwatts_per_cubic_meter

    
    @property
    def deciwatts_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__deciwatts_per_cubic_meter != None:
            return self.__deciwatts_per_cubic_meter
        self.__deciwatts_per_cubic_meter = self.__convert_from_base(PowerDensityUnits.DeciwattPerCubicMeter)
        return self.__deciwatts_per_cubic_meter

    
    @property
    def decawatts_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__decawatts_per_cubic_meter != None:
            return self.__decawatts_per_cubic_meter
        self.__decawatts_per_cubic_meter = self.__convert_from_base(PowerDensityUnits.DecawattPerCubicMeter)
        return self.__decawatts_per_cubic_meter

    
    @property
    def kilowatts_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__kilowatts_per_cubic_meter != None:
            return self.__kilowatts_per_cubic_meter
        self.__kilowatts_per_cubic_meter = self.__convert_from_base(PowerDensityUnits.KilowattPerCubicMeter)
        return self.__kilowatts_per_cubic_meter

    
    @property
    def megawatts_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__megawatts_per_cubic_meter != None:
            return self.__megawatts_per_cubic_meter
        self.__megawatts_per_cubic_meter = self.__convert_from_base(PowerDensityUnits.MegawattPerCubicMeter)
        return self.__megawatts_per_cubic_meter

    
    @property
    def gigawatts_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__gigawatts_per_cubic_meter != None:
            return self.__gigawatts_per_cubic_meter
        self.__gigawatts_per_cubic_meter = self.__convert_from_base(PowerDensityUnits.GigawattPerCubicMeter)
        return self.__gigawatts_per_cubic_meter

    
    @property
    def terawatts_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__terawatts_per_cubic_meter != None:
            return self.__terawatts_per_cubic_meter
        self.__terawatts_per_cubic_meter = self.__convert_from_base(PowerDensityUnits.TerawattPerCubicMeter)
        return self.__terawatts_per_cubic_meter

    
    @property
    def picowatts_per_cubic_inch(self) -> float:
        """
        
        """
        if self.__picowatts_per_cubic_inch != None:
            return self.__picowatts_per_cubic_inch
        self.__picowatts_per_cubic_inch = self.__convert_from_base(PowerDensityUnits.PicowattPerCubicInch)
        return self.__picowatts_per_cubic_inch

    
    @property
    def nanowatts_per_cubic_inch(self) -> float:
        """
        
        """
        if self.__nanowatts_per_cubic_inch != None:
            return self.__nanowatts_per_cubic_inch
        self.__nanowatts_per_cubic_inch = self.__convert_from_base(PowerDensityUnits.NanowattPerCubicInch)
        return self.__nanowatts_per_cubic_inch

    
    @property
    def microwatts_per_cubic_inch(self) -> float:
        """
        
        """
        if self.__microwatts_per_cubic_inch != None:
            return self.__microwatts_per_cubic_inch
        self.__microwatts_per_cubic_inch = self.__convert_from_base(PowerDensityUnits.MicrowattPerCubicInch)
        return self.__microwatts_per_cubic_inch

    
    @property
    def milliwatts_per_cubic_inch(self) -> float:
        """
        
        """
        if self.__milliwatts_per_cubic_inch != None:
            return self.__milliwatts_per_cubic_inch
        self.__milliwatts_per_cubic_inch = self.__convert_from_base(PowerDensityUnits.MilliwattPerCubicInch)
        return self.__milliwatts_per_cubic_inch

    
    @property
    def deciwatts_per_cubic_inch(self) -> float:
        """
        
        """
        if self.__deciwatts_per_cubic_inch != None:
            return self.__deciwatts_per_cubic_inch
        self.__deciwatts_per_cubic_inch = self.__convert_from_base(PowerDensityUnits.DeciwattPerCubicInch)
        return self.__deciwatts_per_cubic_inch

    
    @property
    def decawatts_per_cubic_inch(self) -> float:
        """
        
        """
        if self.__decawatts_per_cubic_inch != None:
            return self.__decawatts_per_cubic_inch
        self.__decawatts_per_cubic_inch = self.__convert_from_base(PowerDensityUnits.DecawattPerCubicInch)
        return self.__decawatts_per_cubic_inch

    
    @property
    def kilowatts_per_cubic_inch(self) -> float:
        """
        
        """
        if self.__kilowatts_per_cubic_inch != None:
            return self.__kilowatts_per_cubic_inch
        self.__kilowatts_per_cubic_inch = self.__convert_from_base(PowerDensityUnits.KilowattPerCubicInch)
        return self.__kilowatts_per_cubic_inch

    
    @property
    def megawatts_per_cubic_inch(self) -> float:
        """
        
        """
        if self.__megawatts_per_cubic_inch != None:
            return self.__megawatts_per_cubic_inch
        self.__megawatts_per_cubic_inch = self.__convert_from_base(PowerDensityUnits.MegawattPerCubicInch)
        return self.__megawatts_per_cubic_inch

    
    @property
    def gigawatts_per_cubic_inch(self) -> float:
        """
        
        """
        if self.__gigawatts_per_cubic_inch != None:
            return self.__gigawatts_per_cubic_inch
        self.__gigawatts_per_cubic_inch = self.__convert_from_base(PowerDensityUnits.GigawattPerCubicInch)
        return self.__gigawatts_per_cubic_inch

    
    @property
    def terawatts_per_cubic_inch(self) -> float:
        """
        
        """
        if self.__terawatts_per_cubic_inch != None:
            return self.__terawatts_per_cubic_inch
        self.__terawatts_per_cubic_inch = self.__convert_from_base(PowerDensityUnits.TerawattPerCubicInch)
        return self.__terawatts_per_cubic_inch

    
    @property
    def picowatts_per_cubic_foot(self) -> float:
        """
        
        """
        if self.__picowatts_per_cubic_foot != None:
            return self.__picowatts_per_cubic_foot
        self.__picowatts_per_cubic_foot = self.__convert_from_base(PowerDensityUnits.PicowattPerCubicFoot)
        return self.__picowatts_per_cubic_foot

    
    @property
    def nanowatts_per_cubic_foot(self) -> float:
        """
        
        """
        if self.__nanowatts_per_cubic_foot != None:
            return self.__nanowatts_per_cubic_foot
        self.__nanowatts_per_cubic_foot = self.__convert_from_base(PowerDensityUnits.NanowattPerCubicFoot)
        return self.__nanowatts_per_cubic_foot

    
    @property
    def microwatts_per_cubic_foot(self) -> float:
        """
        
        """
        if self.__microwatts_per_cubic_foot != None:
            return self.__microwatts_per_cubic_foot
        self.__microwatts_per_cubic_foot = self.__convert_from_base(PowerDensityUnits.MicrowattPerCubicFoot)
        return self.__microwatts_per_cubic_foot

    
    @property
    def milliwatts_per_cubic_foot(self) -> float:
        """
        
        """
        if self.__milliwatts_per_cubic_foot != None:
            return self.__milliwatts_per_cubic_foot
        self.__milliwatts_per_cubic_foot = self.__convert_from_base(PowerDensityUnits.MilliwattPerCubicFoot)
        return self.__milliwatts_per_cubic_foot

    
    @property
    def deciwatts_per_cubic_foot(self) -> float:
        """
        
        """
        if self.__deciwatts_per_cubic_foot != None:
            return self.__deciwatts_per_cubic_foot
        self.__deciwatts_per_cubic_foot = self.__convert_from_base(PowerDensityUnits.DeciwattPerCubicFoot)
        return self.__deciwatts_per_cubic_foot

    
    @property
    def decawatts_per_cubic_foot(self) -> float:
        """
        
        """
        if self.__decawatts_per_cubic_foot != None:
            return self.__decawatts_per_cubic_foot
        self.__decawatts_per_cubic_foot = self.__convert_from_base(PowerDensityUnits.DecawattPerCubicFoot)
        return self.__decawatts_per_cubic_foot

    
    @property
    def kilowatts_per_cubic_foot(self) -> float:
        """
        
        """
        if self.__kilowatts_per_cubic_foot != None:
            return self.__kilowatts_per_cubic_foot
        self.__kilowatts_per_cubic_foot = self.__convert_from_base(PowerDensityUnits.KilowattPerCubicFoot)
        return self.__kilowatts_per_cubic_foot

    
    @property
    def megawatts_per_cubic_foot(self) -> float:
        """
        
        """
        if self.__megawatts_per_cubic_foot != None:
            return self.__megawatts_per_cubic_foot
        self.__megawatts_per_cubic_foot = self.__convert_from_base(PowerDensityUnits.MegawattPerCubicFoot)
        return self.__megawatts_per_cubic_foot

    
    @property
    def gigawatts_per_cubic_foot(self) -> float:
        """
        
        """
        if self.__gigawatts_per_cubic_foot != None:
            return self.__gigawatts_per_cubic_foot
        self.__gigawatts_per_cubic_foot = self.__convert_from_base(PowerDensityUnits.GigawattPerCubicFoot)
        return self.__gigawatts_per_cubic_foot

    
    @property
    def terawatts_per_cubic_foot(self) -> float:
        """
        
        """
        if self.__terawatts_per_cubic_foot != None:
            return self.__terawatts_per_cubic_foot
        self.__terawatts_per_cubic_foot = self.__convert_from_base(PowerDensityUnits.TerawattPerCubicFoot)
        return self.__terawatts_per_cubic_foot

    
    @property
    def picowatts_per_liter(self) -> float:
        """
        
        """
        if self.__picowatts_per_liter != None:
            return self.__picowatts_per_liter
        self.__picowatts_per_liter = self.__convert_from_base(PowerDensityUnits.PicowattPerLiter)
        return self.__picowatts_per_liter

    
    @property
    def nanowatts_per_liter(self) -> float:
        """
        
        """
        if self.__nanowatts_per_liter != None:
            return self.__nanowatts_per_liter
        self.__nanowatts_per_liter = self.__convert_from_base(PowerDensityUnits.NanowattPerLiter)
        return self.__nanowatts_per_liter

    
    @property
    def microwatts_per_liter(self) -> float:
        """
        
        """
        if self.__microwatts_per_liter != None:
            return self.__microwatts_per_liter
        self.__microwatts_per_liter = self.__convert_from_base(PowerDensityUnits.MicrowattPerLiter)
        return self.__microwatts_per_liter

    
    @property
    def milliwatts_per_liter(self) -> float:
        """
        
        """
        if self.__milliwatts_per_liter != None:
            return self.__milliwatts_per_liter
        self.__milliwatts_per_liter = self.__convert_from_base(PowerDensityUnits.MilliwattPerLiter)
        return self.__milliwatts_per_liter

    
    @property
    def deciwatts_per_liter(self) -> float:
        """
        
        """
        if self.__deciwatts_per_liter != None:
            return self.__deciwatts_per_liter
        self.__deciwatts_per_liter = self.__convert_from_base(PowerDensityUnits.DeciwattPerLiter)
        return self.__deciwatts_per_liter

    
    @property
    def decawatts_per_liter(self) -> float:
        """
        
        """
        if self.__decawatts_per_liter != None:
            return self.__decawatts_per_liter
        self.__decawatts_per_liter = self.__convert_from_base(PowerDensityUnits.DecawattPerLiter)
        return self.__decawatts_per_liter

    
    @property
    def kilowatts_per_liter(self) -> float:
        """
        
        """
        if self.__kilowatts_per_liter != None:
            return self.__kilowatts_per_liter
        self.__kilowatts_per_liter = self.__convert_from_base(PowerDensityUnits.KilowattPerLiter)
        return self.__kilowatts_per_liter

    
    @property
    def megawatts_per_liter(self) -> float:
        """
        
        """
        if self.__megawatts_per_liter != None:
            return self.__megawatts_per_liter
        self.__megawatts_per_liter = self.__convert_from_base(PowerDensityUnits.MegawattPerLiter)
        return self.__megawatts_per_liter

    
    @property
    def gigawatts_per_liter(self) -> float:
        """
        
        """
        if self.__gigawatts_per_liter != None:
            return self.__gigawatts_per_liter
        self.__gigawatts_per_liter = self.__convert_from_base(PowerDensityUnits.GigawattPerLiter)
        return self.__gigawatts_per_liter

    
    @property
    def terawatts_per_liter(self) -> float:
        """
        
        """
        if self.__terawatts_per_liter != None:
            return self.__terawatts_per_liter
        self.__terawatts_per_liter = self.__convert_from_base(PowerDensityUnits.TerawattPerLiter)
        return self.__terawatts_per_liter

    
    def to_string(self, unit: PowerDensityUnits = PowerDensityUnits.WattPerCubicMeter) -> string:
        """
        Format the PowerDensity to string.
        Note! the default format for PowerDensity is WattPerCubicMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == PowerDensityUnits.WattPerCubicMeter:
            return f"""{self.watts_per_cubic_meter} W/m³"""
        
        if unit == PowerDensityUnits.WattPerCubicInch:
            return f"""{self.watts_per_cubic_inch} W/in³"""
        
        if unit == PowerDensityUnits.WattPerCubicFoot:
            return f"""{self.watts_per_cubic_foot} W/ft³"""
        
        if unit == PowerDensityUnits.WattPerLiter:
            return f"""{self.watts_per_liter} W/l"""
        
        if unit == PowerDensityUnits.PicowattPerCubicMeter:
            return f"""{self.picowatts_per_cubic_meter} """
        
        if unit == PowerDensityUnits.NanowattPerCubicMeter:
            return f"""{self.nanowatts_per_cubic_meter} """
        
        if unit == PowerDensityUnits.MicrowattPerCubicMeter:
            return f"""{self.microwatts_per_cubic_meter} """
        
        if unit == PowerDensityUnits.MilliwattPerCubicMeter:
            return f"""{self.milliwatts_per_cubic_meter} """
        
        if unit == PowerDensityUnits.DeciwattPerCubicMeter:
            return f"""{self.deciwatts_per_cubic_meter} """
        
        if unit == PowerDensityUnits.DecawattPerCubicMeter:
            return f"""{self.decawatts_per_cubic_meter} """
        
        if unit == PowerDensityUnits.KilowattPerCubicMeter:
            return f"""{self.kilowatts_per_cubic_meter} """
        
        if unit == PowerDensityUnits.MegawattPerCubicMeter:
            return f"""{self.megawatts_per_cubic_meter} """
        
        if unit == PowerDensityUnits.GigawattPerCubicMeter:
            return f"""{self.gigawatts_per_cubic_meter} """
        
        if unit == PowerDensityUnits.TerawattPerCubicMeter:
            return f"""{self.terawatts_per_cubic_meter} """
        
        if unit == PowerDensityUnits.PicowattPerCubicInch:
            return f"""{self.picowatts_per_cubic_inch} """
        
        if unit == PowerDensityUnits.NanowattPerCubicInch:
            return f"""{self.nanowatts_per_cubic_inch} """
        
        if unit == PowerDensityUnits.MicrowattPerCubicInch:
            return f"""{self.microwatts_per_cubic_inch} """
        
        if unit == PowerDensityUnits.MilliwattPerCubicInch:
            return f"""{self.milliwatts_per_cubic_inch} """
        
        if unit == PowerDensityUnits.DeciwattPerCubicInch:
            return f"""{self.deciwatts_per_cubic_inch} """
        
        if unit == PowerDensityUnits.DecawattPerCubicInch:
            return f"""{self.decawatts_per_cubic_inch} """
        
        if unit == PowerDensityUnits.KilowattPerCubicInch:
            return f"""{self.kilowatts_per_cubic_inch} """
        
        if unit == PowerDensityUnits.MegawattPerCubicInch:
            return f"""{self.megawatts_per_cubic_inch} """
        
        if unit == PowerDensityUnits.GigawattPerCubicInch:
            return f"""{self.gigawatts_per_cubic_inch} """
        
        if unit == PowerDensityUnits.TerawattPerCubicInch:
            return f"""{self.terawatts_per_cubic_inch} """
        
        if unit == PowerDensityUnits.PicowattPerCubicFoot:
            return f"""{self.picowatts_per_cubic_foot} """
        
        if unit == PowerDensityUnits.NanowattPerCubicFoot:
            return f"""{self.nanowatts_per_cubic_foot} """
        
        if unit == PowerDensityUnits.MicrowattPerCubicFoot:
            return f"""{self.microwatts_per_cubic_foot} """
        
        if unit == PowerDensityUnits.MilliwattPerCubicFoot:
            return f"""{self.milliwatts_per_cubic_foot} """
        
        if unit == PowerDensityUnits.DeciwattPerCubicFoot:
            return f"""{self.deciwatts_per_cubic_foot} """
        
        if unit == PowerDensityUnits.DecawattPerCubicFoot:
            return f"""{self.decawatts_per_cubic_foot} """
        
        if unit == PowerDensityUnits.KilowattPerCubicFoot:
            return f"""{self.kilowatts_per_cubic_foot} """
        
        if unit == PowerDensityUnits.MegawattPerCubicFoot:
            return f"""{self.megawatts_per_cubic_foot} """
        
        if unit == PowerDensityUnits.GigawattPerCubicFoot:
            return f"""{self.gigawatts_per_cubic_foot} """
        
        if unit == PowerDensityUnits.TerawattPerCubicFoot:
            return f"""{self.terawatts_per_cubic_foot} """
        
        if unit == PowerDensityUnits.PicowattPerLiter:
            return f"""{self.picowatts_per_liter} """
        
        if unit == PowerDensityUnits.NanowattPerLiter:
            return f"""{self.nanowatts_per_liter} """
        
        if unit == PowerDensityUnits.MicrowattPerLiter:
            return f"""{self.microwatts_per_liter} """
        
        if unit == PowerDensityUnits.MilliwattPerLiter:
            return f"""{self.milliwatts_per_liter} """
        
        if unit == PowerDensityUnits.DeciwattPerLiter:
            return f"""{self.deciwatts_per_liter} """
        
        if unit == PowerDensityUnits.DecawattPerLiter:
            return f"""{self.decawatts_per_liter} """
        
        if unit == PowerDensityUnits.KilowattPerLiter:
            return f"""{self.kilowatts_per_liter} """
        
        if unit == PowerDensityUnits.MegawattPerLiter:
            return f"""{self.megawatts_per_liter} """
        
        if unit == PowerDensityUnits.GigawattPerLiter:
            return f"""{self.gigawatts_per_liter} """
        
        if unit == PowerDensityUnits.TerawattPerLiter:
            return f"""{self.terawatts_per_liter} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: PowerDensityUnits = PowerDensityUnits.WattPerCubicMeter) -> string:
        """
        Get PowerDensity unit abbreviation.
        Note! the default abbreviation for PowerDensity is WattPerCubicMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == PowerDensityUnits.WattPerCubicMeter:
            return """W/m³"""
        
        if unit_abbreviation == PowerDensityUnits.WattPerCubicInch:
            return """W/in³"""
        
        if unit_abbreviation == PowerDensityUnits.WattPerCubicFoot:
            return """W/ft³"""
        
        if unit_abbreviation == PowerDensityUnits.WattPerLiter:
            return """W/l"""
        
        if unit_abbreviation == PowerDensityUnits.PicowattPerCubicMeter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.NanowattPerCubicMeter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.MicrowattPerCubicMeter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.MilliwattPerCubicMeter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.DeciwattPerCubicMeter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.DecawattPerCubicMeter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.KilowattPerCubicMeter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.MegawattPerCubicMeter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.GigawattPerCubicMeter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.TerawattPerCubicMeter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.PicowattPerCubicInch:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.NanowattPerCubicInch:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.MicrowattPerCubicInch:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.MilliwattPerCubicInch:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.DeciwattPerCubicInch:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.DecawattPerCubicInch:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.KilowattPerCubicInch:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.MegawattPerCubicInch:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.GigawattPerCubicInch:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.TerawattPerCubicInch:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.PicowattPerCubicFoot:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.NanowattPerCubicFoot:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.MicrowattPerCubicFoot:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.MilliwattPerCubicFoot:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.DeciwattPerCubicFoot:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.DecawattPerCubicFoot:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.KilowattPerCubicFoot:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.MegawattPerCubicFoot:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.GigawattPerCubicFoot:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.TerawattPerCubicFoot:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.PicowattPerLiter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.NanowattPerLiter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.MicrowattPerLiter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.MilliwattPerLiter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.DeciwattPerLiter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.DecawattPerLiter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.KilowattPerLiter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.MegawattPerLiter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.GigawattPerLiter:
            return """"""
        
        if unit_abbreviation == PowerDensityUnits.TerawattPerLiter:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, PowerDensity):
            raise TypeError("unsupported operand type(s) for +: 'PowerDensity' and '{}'".format(type(other).__name__))
        return PowerDensity(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, PowerDensity):
            raise TypeError("unsupported operand type(s) for *: 'PowerDensity' and '{}'".format(type(other).__name__))
        return PowerDensity(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, PowerDensity):
            raise TypeError("unsupported operand type(s) for -: 'PowerDensity' and '{}'".format(type(other).__name__))
        return PowerDensity(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, PowerDensity):
            raise TypeError("unsupported operand type(s) for /: 'PowerDensity' and '{}'".format(type(other).__name__))
        return PowerDensity(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, PowerDensity):
            raise TypeError("unsupported operand type(s) for %: 'PowerDensity' and '{}'".format(type(other).__name__))
        return PowerDensity(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, PowerDensity):
            raise TypeError("unsupported operand type(s) for **: 'PowerDensity' and '{}'".format(type(other).__name__))
        return PowerDensity(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, PowerDensity):
            raise TypeError("unsupported operand type(s) for ==: 'PowerDensity' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, PowerDensity):
            raise TypeError("unsupported operand type(s) for <: 'PowerDensity' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, PowerDensity):
            raise TypeError("unsupported operand type(s) for >: 'PowerDensity' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, PowerDensity):
            raise TypeError("unsupported operand type(s) for <=: 'PowerDensity' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, PowerDensity):
            raise TypeError("unsupported operand type(s) for >=: 'PowerDensity' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class TurbidityUnits(Enum):
        """
            TurbidityUnits enumeration
        """
        
        NTU = 'ntu'
        """
            
        """
        

class Turbidity:
    """
    Turbidity is the cloudiness or haziness of a fluid caused by large numbers of individual particles that are generally invisible to the naked eye, similar to smoke in air. The measurement of turbidity is a key test of water quality.

    Args:
        value (float): The value.
        from_unit (TurbidityUnits): The Turbidity unit to create from, The default unit is NTU
    """
    def __init__(self, value: float, from_unit: TurbidityUnits = TurbidityUnits.NTU):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__ntu = None
        

    def __convert_from_base(self, from_unit: TurbidityUnits) -> float:
        value = self.__value
        
        if from_unit == TurbidityUnits.NTU:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: TurbidityUnits) -> float:
        
        if to_unit == TurbidityUnits.NTU:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_ntu(ntu: float):
        """
        Create a new instance of Turbidity from a value in ntu.

        

        :param meters: The Turbidity value in ntu.
        :type ntu: float
        :return: A new instance of Turbidity.
        :rtype: Turbidity
        """
        return Turbidity(ntu, TurbidityUnits.NTU)

    
    @property
    def ntu(self) -> float:
        """
        
        """
        if self.__ntu != None:
            return self.__ntu
        self.__ntu = self.__convert_from_base(TurbidityUnits.NTU)
        return self.__ntu

    
    def to_string(self, unit: TurbidityUnits = TurbidityUnits.NTU) -> string:
        """
        Format the Turbidity to string.
        Note! the default format for Turbidity is NTU.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == TurbidityUnits.NTU:
            return f"""{self.ntu} NTU"""
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: TurbidityUnits = TurbidityUnits.NTU) -> string:
        """
        Get Turbidity unit abbreviation.
        Note! the default abbreviation for Turbidity is NTU.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == TurbidityUnits.NTU:
            return """NTU"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Turbidity):
            raise TypeError("unsupported operand type(s) for +: 'Turbidity' and '{}'".format(type(other).__name__))
        return Turbidity(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Turbidity):
            raise TypeError("unsupported operand type(s) for *: 'Turbidity' and '{}'".format(type(other).__name__))
        return Turbidity(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Turbidity):
            raise TypeError("unsupported operand type(s) for -: 'Turbidity' and '{}'".format(type(other).__name__))
        return Turbidity(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Turbidity):
            raise TypeError("unsupported operand type(s) for /: 'Turbidity' and '{}'".format(type(other).__name__))
        return Turbidity(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Turbidity):
            raise TypeError("unsupported operand type(s) for %: 'Turbidity' and '{}'".format(type(other).__name__))
        return Turbidity(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Turbidity):
            raise TypeError("unsupported operand type(s) for **: 'Turbidity' and '{}'".format(type(other).__name__))
        return Turbidity(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Turbidity):
            raise TypeError("unsupported operand type(s) for ==: 'Turbidity' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Turbidity):
            raise TypeError("unsupported operand type(s) for <: 'Turbidity' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Turbidity):
            raise TypeError("unsupported operand type(s) for >: 'Turbidity' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Turbidity):
            raise TypeError("unsupported operand type(s) for <=: 'Turbidity' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Turbidity):
            raise TypeError("unsupported operand type(s) for >=: 'Turbidity' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class ElectricResistanceUnits(Enum):
        """
            ElectricResistanceUnits enumeration
        """
        
        Ohm = 'ohm'
        """
            
        """
        
        Microohm = 'microohm'
        """
            
        """
        
        Milliohm = 'milliohm'
        """
            
        """
        
        Kiloohm = 'kiloohm'
        """
            
        """
        
        Megaohm = 'megaohm'
        """
            
        """
        
        Gigaohm = 'gigaohm'
        """
            
        """
        
        Teraohm = 'teraohm'
        """
            
        """
        

class ElectricResistance:
    """
    The electrical resistance of an electrical conductor is the opposition to the passage of an electric current through that conductor.

    Args:
        value (float): The value.
        from_unit (ElectricResistanceUnits): The ElectricResistance unit to create from, The default unit is Ohm
    """
    def __init__(self, value: float, from_unit: ElectricResistanceUnits = ElectricResistanceUnits.Ohm):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__ohms = None
        
        self.__microohms = None
        
        self.__milliohms = None
        
        self.__kiloohms = None
        
        self.__megaohms = None
        
        self.__gigaohms = None
        
        self.__teraohms = None
        

    def __convert_from_base(self, from_unit: ElectricResistanceUnits) -> float:
        value = self.__value
        
        if from_unit == ElectricResistanceUnits.Ohm:
            return (value)
        
        if from_unit == ElectricResistanceUnits.Microohm:
            return ((value) / 1e-06)
        
        if from_unit == ElectricResistanceUnits.Milliohm:
            return ((value) / 0.001)
        
        if from_unit == ElectricResistanceUnits.Kiloohm:
            return ((value) / 1000.0)
        
        if from_unit == ElectricResistanceUnits.Megaohm:
            return ((value) / 1000000.0)
        
        if from_unit == ElectricResistanceUnits.Gigaohm:
            return ((value) / 1000000000.0)
        
        if from_unit == ElectricResistanceUnits.Teraohm:
            return ((value) / 1000000000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricResistanceUnits) -> float:
        
        if to_unit == ElectricResistanceUnits.Ohm:
            return (value)
        
        if to_unit == ElectricResistanceUnits.Microohm:
            return ((value) * 1e-06)
        
        if to_unit == ElectricResistanceUnits.Milliohm:
            return ((value) * 0.001)
        
        if to_unit == ElectricResistanceUnits.Kiloohm:
            return ((value) * 1000.0)
        
        if to_unit == ElectricResistanceUnits.Megaohm:
            return ((value) * 1000000.0)
        
        if to_unit == ElectricResistanceUnits.Gigaohm:
            return ((value) * 1000000000.0)
        
        if to_unit == ElectricResistanceUnits.Teraohm:
            return ((value) * 1000000000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_ohms(ohms: float):
        """
        Create a new instance of ElectricResistance from a value in ohms.

        

        :param meters: The ElectricResistance value in ohms.
        :type ohms: float
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance(ohms, ElectricResistanceUnits.Ohm)

    
    @staticmethod
    def from_microohms(microohms: float):
        """
        Create a new instance of ElectricResistance from a value in microohms.

        

        :param meters: The ElectricResistance value in microohms.
        :type microohms: float
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance(microohms, ElectricResistanceUnits.Microohm)

    
    @staticmethod
    def from_milliohms(milliohms: float):
        """
        Create a new instance of ElectricResistance from a value in milliohms.

        

        :param meters: The ElectricResistance value in milliohms.
        :type milliohms: float
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance(milliohms, ElectricResistanceUnits.Milliohm)

    
    @staticmethod
    def from_kiloohms(kiloohms: float):
        """
        Create a new instance of ElectricResistance from a value in kiloohms.

        

        :param meters: The ElectricResistance value in kiloohms.
        :type kiloohms: float
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance(kiloohms, ElectricResistanceUnits.Kiloohm)

    
    @staticmethod
    def from_megaohms(megaohms: float):
        """
        Create a new instance of ElectricResistance from a value in megaohms.

        

        :param meters: The ElectricResistance value in megaohms.
        :type megaohms: float
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance(megaohms, ElectricResistanceUnits.Megaohm)

    
    @staticmethod
    def from_gigaohms(gigaohms: float):
        """
        Create a new instance of ElectricResistance from a value in gigaohms.

        

        :param meters: The ElectricResistance value in gigaohms.
        :type gigaohms: float
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance(gigaohms, ElectricResistanceUnits.Gigaohm)

    
    @staticmethod
    def from_teraohms(teraohms: float):
        """
        Create a new instance of ElectricResistance from a value in teraohms.

        

        :param meters: The ElectricResistance value in teraohms.
        :type teraohms: float
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance(teraohms, ElectricResistanceUnits.Teraohm)

    
    @property
    def ohms(self) -> float:
        """
        
        """
        if self.__ohms != None:
            return self.__ohms
        self.__ohms = self.__convert_from_base(ElectricResistanceUnits.Ohm)
        return self.__ohms

    
    @property
    def microohms(self) -> float:
        """
        
        """
        if self.__microohms != None:
            return self.__microohms
        self.__microohms = self.__convert_from_base(ElectricResistanceUnits.Microohm)
        return self.__microohms

    
    @property
    def milliohms(self) -> float:
        """
        
        """
        if self.__milliohms != None:
            return self.__milliohms
        self.__milliohms = self.__convert_from_base(ElectricResistanceUnits.Milliohm)
        return self.__milliohms

    
    @property
    def kiloohms(self) -> float:
        """
        
        """
        if self.__kiloohms != None:
            return self.__kiloohms
        self.__kiloohms = self.__convert_from_base(ElectricResistanceUnits.Kiloohm)
        return self.__kiloohms

    
    @property
    def megaohms(self) -> float:
        """
        
        """
        if self.__megaohms != None:
            return self.__megaohms
        self.__megaohms = self.__convert_from_base(ElectricResistanceUnits.Megaohm)
        return self.__megaohms

    
    @property
    def gigaohms(self) -> float:
        """
        
        """
        if self.__gigaohms != None:
            return self.__gigaohms
        self.__gigaohms = self.__convert_from_base(ElectricResistanceUnits.Gigaohm)
        return self.__gigaohms

    
    @property
    def teraohms(self) -> float:
        """
        
        """
        if self.__teraohms != None:
            return self.__teraohms
        self.__teraohms = self.__convert_from_base(ElectricResistanceUnits.Teraohm)
        return self.__teraohms

    
    def to_string(self, unit: ElectricResistanceUnits = ElectricResistanceUnits.Ohm) -> string:
        """
        Format the ElectricResistance to string.
        Note! the default format for ElectricResistance is Ohm.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == ElectricResistanceUnits.Ohm:
            return f"""{self.ohms} Ω"""
        
        if unit == ElectricResistanceUnits.Microohm:
            return f"""{self.microohms} """
        
        if unit == ElectricResistanceUnits.Milliohm:
            return f"""{self.milliohms} """
        
        if unit == ElectricResistanceUnits.Kiloohm:
            return f"""{self.kiloohms} """
        
        if unit == ElectricResistanceUnits.Megaohm:
            return f"""{self.megaohms} """
        
        if unit == ElectricResistanceUnits.Gigaohm:
            return f"""{self.gigaohms} """
        
        if unit == ElectricResistanceUnits.Teraohm:
            return f"""{self.teraohms} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricResistanceUnits = ElectricResistanceUnits.Ohm) -> string:
        """
        Get ElectricResistance unit abbreviation.
        Note! the default abbreviation for ElectricResistance is Ohm.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricResistanceUnits.Ohm:
            return """Ω"""
        
        if unit_abbreviation == ElectricResistanceUnits.Microohm:
            return """"""
        
        if unit_abbreviation == ElectricResistanceUnits.Milliohm:
            return """"""
        
        if unit_abbreviation == ElectricResistanceUnits.Kiloohm:
            return """"""
        
        if unit_abbreviation == ElectricResistanceUnits.Megaohm:
            return """"""
        
        if unit_abbreviation == ElectricResistanceUnits.Gigaohm:
            return """"""
        
        if unit_abbreviation == ElectricResistanceUnits.Teraohm:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, ElectricResistance):
            raise TypeError("unsupported operand type(s) for +: 'ElectricResistance' and '{}'".format(type(other).__name__))
        return ElectricResistance(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, ElectricResistance):
            raise TypeError("unsupported operand type(s) for *: 'ElectricResistance' and '{}'".format(type(other).__name__))
        return ElectricResistance(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, ElectricResistance):
            raise TypeError("unsupported operand type(s) for -: 'ElectricResistance' and '{}'".format(type(other).__name__))
        return ElectricResistance(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, ElectricResistance):
            raise TypeError("unsupported operand type(s) for /: 'ElectricResistance' and '{}'".format(type(other).__name__))
        return ElectricResistance(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, ElectricResistance):
            raise TypeError("unsupported operand type(s) for %: 'ElectricResistance' and '{}'".format(type(other).__name__))
        return ElectricResistance(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, ElectricResistance):
            raise TypeError("unsupported operand type(s) for **: 'ElectricResistance' and '{}'".format(type(other).__name__))
        return ElectricResistance(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, ElectricResistance):
            raise TypeError("unsupported operand type(s) for ==: 'ElectricResistance' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, ElectricResistance):
            raise TypeError("unsupported operand type(s) for <: 'ElectricResistance' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, ElectricResistance):
            raise TypeError("unsupported operand type(s) for >: 'ElectricResistance' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, ElectricResistance):
            raise TypeError("unsupported operand type(s) for <=: 'ElectricResistance' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, ElectricResistance):
            raise TypeError("unsupported operand type(s) for >=: 'ElectricResistance' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class ElectricPotentialDcUnits(Enum):
        """
            ElectricPotentialDcUnits enumeration
        """
        
        VoltDc = 'volt_dc'
        """
            
        """
        
        MicrovoltDc = 'microvolt_dc'
        """
            
        """
        
        MillivoltDc = 'millivolt_dc'
        """
            
        """
        
        KilovoltDc = 'kilovolt_dc'
        """
            
        """
        
        MegavoltDc = 'megavolt_dc'
        """
            
        """
        

class ElectricPotentialDc:
    """
    The Electric Potential of a system known to use Direct Current.

    Args:
        value (float): The value.
        from_unit (ElectricPotentialDcUnits): The ElectricPotentialDc unit to create from, The default unit is VoltDc
    """
    def __init__(self, value: float, from_unit: ElectricPotentialDcUnits = ElectricPotentialDcUnits.VoltDc):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__volts_dc = None
        
        self.__microvolts_dc = None
        
        self.__millivolts_dc = None
        
        self.__kilovolts_dc = None
        
        self.__megavolts_dc = None
        

    def __convert_from_base(self, from_unit: ElectricPotentialDcUnits) -> float:
        value = self.__value
        
        if from_unit == ElectricPotentialDcUnits.VoltDc:
            return (value)
        
        if from_unit == ElectricPotentialDcUnits.MicrovoltDc:
            return ((value) / 1e-06)
        
        if from_unit == ElectricPotentialDcUnits.MillivoltDc:
            return ((value) / 0.001)
        
        if from_unit == ElectricPotentialDcUnits.KilovoltDc:
            return ((value) / 1000.0)
        
        if from_unit == ElectricPotentialDcUnits.MegavoltDc:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricPotentialDcUnits) -> float:
        
        if to_unit == ElectricPotentialDcUnits.VoltDc:
            return (value)
        
        if to_unit == ElectricPotentialDcUnits.MicrovoltDc:
            return ((value) * 1e-06)
        
        if to_unit == ElectricPotentialDcUnits.MillivoltDc:
            return ((value) * 0.001)
        
        if to_unit == ElectricPotentialDcUnits.KilovoltDc:
            return ((value) * 1000.0)
        
        if to_unit == ElectricPotentialDcUnits.MegavoltDc:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_volts_dc(volts_dc: float):
        """
        Create a new instance of ElectricPotentialDc from a value in volts_dc.

        

        :param meters: The ElectricPotentialDc value in volts_dc.
        :type volts_dc: float
        :return: A new instance of ElectricPotentialDc.
        :rtype: ElectricPotentialDc
        """
        return ElectricPotentialDc(volts_dc, ElectricPotentialDcUnits.VoltDc)

    
    @staticmethod
    def from_microvolts_dc(microvolts_dc: float):
        """
        Create a new instance of ElectricPotentialDc from a value in microvolts_dc.

        

        :param meters: The ElectricPotentialDc value in microvolts_dc.
        :type microvolts_dc: float
        :return: A new instance of ElectricPotentialDc.
        :rtype: ElectricPotentialDc
        """
        return ElectricPotentialDc(microvolts_dc, ElectricPotentialDcUnits.MicrovoltDc)

    
    @staticmethod
    def from_millivolts_dc(millivolts_dc: float):
        """
        Create a new instance of ElectricPotentialDc from a value in millivolts_dc.

        

        :param meters: The ElectricPotentialDc value in millivolts_dc.
        :type millivolts_dc: float
        :return: A new instance of ElectricPotentialDc.
        :rtype: ElectricPotentialDc
        """
        return ElectricPotentialDc(millivolts_dc, ElectricPotentialDcUnits.MillivoltDc)

    
    @staticmethod
    def from_kilovolts_dc(kilovolts_dc: float):
        """
        Create a new instance of ElectricPotentialDc from a value in kilovolts_dc.

        

        :param meters: The ElectricPotentialDc value in kilovolts_dc.
        :type kilovolts_dc: float
        :return: A new instance of ElectricPotentialDc.
        :rtype: ElectricPotentialDc
        """
        return ElectricPotentialDc(kilovolts_dc, ElectricPotentialDcUnits.KilovoltDc)

    
    @staticmethod
    def from_megavolts_dc(megavolts_dc: float):
        """
        Create a new instance of ElectricPotentialDc from a value in megavolts_dc.

        

        :param meters: The ElectricPotentialDc value in megavolts_dc.
        :type megavolts_dc: float
        :return: A new instance of ElectricPotentialDc.
        :rtype: ElectricPotentialDc
        """
        return ElectricPotentialDc(megavolts_dc, ElectricPotentialDcUnits.MegavoltDc)

    
    @property
    def volts_dc(self) -> float:
        """
        
        """
        if self.__volts_dc != None:
            return self.__volts_dc
        self.__volts_dc = self.__convert_from_base(ElectricPotentialDcUnits.VoltDc)
        return self.__volts_dc

    
    @property
    def microvolts_dc(self) -> float:
        """
        
        """
        if self.__microvolts_dc != None:
            return self.__microvolts_dc
        self.__microvolts_dc = self.__convert_from_base(ElectricPotentialDcUnits.MicrovoltDc)
        return self.__microvolts_dc

    
    @property
    def millivolts_dc(self) -> float:
        """
        
        """
        if self.__millivolts_dc != None:
            return self.__millivolts_dc
        self.__millivolts_dc = self.__convert_from_base(ElectricPotentialDcUnits.MillivoltDc)
        return self.__millivolts_dc

    
    @property
    def kilovolts_dc(self) -> float:
        """
        
        """
        if self.__kilovolts_dc != None:
            return self.__kilovolts_dc
        self.__kilovolts_dc = self.__convert_from_base(ElectricPotentialDcUnits.KilovoltDc)
        return self.__kilovolts_dc

    
    @property
    def megavolts_dc(self) -> float:
        """
        
        """
        if self.__megavolts_dc != None:
            return self.__megavolts_dc
        self.__megavolts_dc = self.__convert_from_base(ElectricPotentialDcUnits.MegavoltDc)
        return self.__megavolts_dc

    
    def to_string(self, unit: ElectricPotentialDcUnits = ElectricPotentialDcUnits.VoltDc) -> string:
        """
        Format the ElectricPotentialDc to string.
        Note! the default format for ElectricPotentialDc is VoltDc.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == ElectricPotentialDcUnits.VoltDc:
            return f"""{self.volts_dc} Vdc"""
        
        if unit == ElectricPotentialDcUnits.MicrovoltDc:
            return f"""{self.microvolts_dc} """
        
        if unit == ElectricPotentialDcUnits.MillivoltDc:
            return f"""{self.millivolts_dc} """
        
        if unit == ElectricPotentialDcUnits.KilovoltDc:
            return f"""{self.kilovolts_dc} """
        
        if unit == ElectricPotentialDcUnits.MegavoltDc:
            return f"""{self.megavolts_dc} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricPotentialDcUnits = ElectricPotentialDcUnits.VoltDc) -> string:
        """
        Get ElectricPotentialDc unit abbreviation.
        Note! the default abbreviation for ElectricPotentialDc is VoltDc.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricPotentialDcUnits.VoltDc:
            return """Vdc"""
        
        if unit_abbreviation == ElectricPotentialDcUnits.MicrovoltDc:
            return """"""
        
        if unit_abbreviation == ElectricPotentialDcUnits.MillivoltDc:
            return """"""
        
        if unit_abbreviation == ElectricPotentialDcUnits.KilovoltDc:
            return """"""
        
        if unit_abbreviation == ElectricPotentialDcUnits.MegavoltDc:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, ElectricPotentialDc):
            raise TypeError("unsupported operand type(s) for +: 'ElectricPotentialDc' and '{}'".format(type(other).__name__))
        return ElectricPotentialDc(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, ElectricPotentialDc):
            raise TypeError("unsupported operand type(s) for *: 'ElectricPotentialDc' and '{}'".format(type(other).__name__))
        return ElectricPotentialDc(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, ElectricPotentialDc):
            raise TypeError("unsupported operand type(s) for -: 'ElectricPotentialDc' and '{}'".format(type(other).__name__))
        return ElectricPotentialDc(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, ElectricPotentialDc):
            raise TypeError("unsupported operand type(s) for /: 'ElectricPotentialDc' and '{}'".format(type(other).__name__))
        return ElectricPotentialDc(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, ElectricPotentialDc):
            raise TypeError("unsupported operand type(s) for %: 'ElectricPotentialDc' and '{}'".format(type(other).__name__))
        return ElectricPotentialDc(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, ElectricPotentialDc):
            raise TypeError("unsupported operand type(s) for **: 'ElectricPotentialDc' and '{}'".format(type(other).__name__))
        return ElectricPotentialDc(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, ElectricPotentialDc):
            raise TypeError("unsupported operand type(s) for ==: 'ElectricPotentialDc' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, ElectricPotentialDc):
            raise TypeError("unsupported operand type(s) for <: 'ElectricPotentialDc' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, ElectricPotentialDc):
            raise TypeError("unsupported operand type(s) for >: 'ElectricPotentialDc' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, ElectricPotentialDc):
            raise TypeError("unsupported operand type(s) for <=: 'ElectricPotentialDc' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, ElectricPotentialDc):
            raise TypeError("unsupported operand type(s) for >=: 'ElectricPotentialDc' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class IlluminanceUnits(Enum):
        """
            IlluminanceUnits enumeration
        """
        
        Lux = 'lux'
        """
            
        """
        
        Millilux = 'millilux'
        """
            
        """
        
        Kilolux = 'kilolux'
        """
            
        """
        
        Megalux = 'megalux'
        """
            
        """
        

class Illuminance:
    """
    In photometry, illuminance is the total luminous flux incident on a surface, per unit area.

    Args:
        value (float): The value.
        from_unit (IlluminanceUnits): The Illuminance unit to create from, The default unit is Lux
    """
    def __init__(self, value: float, from_unit: IlluminanceUnits = IlluminanceUnits.Lux):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__lux = None
        
        self.__millilux = None
        
        self.__kilolux = None
        
        self.__megalux = None
        

    def __convert_from_base(self, from_unit: IlluminanceUnits) -> float:
        value = self.__value
        
        if from_unit == IlluminanceUnits.Lux:
            return (value)
        
        if from_unit == IlluminanceUnits.Millilux:
            return ((value) / 0.001)
        
        if from_unit == IlluminanceUnits.Kilolux:
            return ((value) / 1000.0)
        
        if from_unit == IlluminanceUnits.Megalux:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: IlluminanceUnits) -> float:
        
        if to_unit == IlluminanceUnits.Lux:
            return (value)
        
        if to_unit == IlluminanceUnits.Millilux:
            return ((value) * 0.001)
        
        if to_unit == IlluminanceUnits.Kilolux:
            return ((value) * 1000.0)
        
        if to_unit == IlluminanceUnits.Megalux:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_lux(lux: float):
        """
        Create a new instance of Illuminance from a value in lux.

        

        :param meters: The Illuminance value in lux.
        :type lux: float
        :return: A new instance of Illuminance.
        :rtype: Illuminance
        """
        return Illuminance(lux, IlluminanceUnits.Lux)

    
    @staticmethod
    def from_millilux(millilux: float):
        """
        Create a new instance of Illuminance from a value in millilux.

        

        :param meters: The Illuminance value in millilux.
        :type millilux: float
        :return: A new instance of Illuminance.
        :rtype: Illuminance
        """
        return Illuminance(millilux, IlluminanceUnits.Millilux)

    
    @staticmethod
    def from_kilolux(kilolux: float):
        """
        Create a new instance of Illuminance from a value in kilolux.

        

        :param meters: The Illuminance value in kilolux.
        :type kilolux: float
        :return: A new instance of Illuminance.
        :rtype: Illuminance
        """
        return Illuminance(kilolux, IlluminanceUnits.Kilolux)

    
    @staticmethod
    def from_megalux(megalux: float):
        """
        Create a new instance of Illuminance from a value in megalux.

        

        :param meters: The Illuminance value in megalux.
        :type megalux: float
        :return: A new instance of Illuminance.
        :rtype: Illuminance
        """
        return Illuminance(megalux, IlluminanceUnits.Megalux)

    
    @property
    def lux(self) -> float:
        """
        
        """
        if self.__lux != None:
            return self.__lux
        self.__lux = self.__convert_from_base(IlluminanceUnits.Lux)
        return self.__lux

    
    @property
    def millilux(self) -> float:
        """
        
        """
        if self.__millilux != None:
            return self.__millilux
        self.__millilux = self.__convert_from_base(IlluminanceUnits.Millilux)
        return self.__millilux

    
    @property
    def kilolux(self) -> float:
        """
        
        """
        if self.__kilolux != None:
            return self.__kilolux
        self.__kilolux = self.__convert_from_base(IlluminanceUnits.Kilolux)
        return self.__kilolux

    
    @property
    def megalux(self) -> float:
        """
        
        """
        if self.__megalux != None:
            return self.__megalux
        self.__megalux = self.__convert_from_base(IlluminanceUnits.Megalux)
        return self.__megalux

    
    def to_string(self, unit: IlluminanceUnits = IlluminanceUnits.Lux) -> string:
        """
        Format the Illuminance to string.
        Note! the default format for Illuminance is Lux.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == IlluminanceUnits.Lux:
            return f"""{self.lux} lx"""
        
        if unit == IlluminanceUnits.Millilux:
            return f"""{self.millilux} """
        
        if unit == IlluminanceUnits.Kilolux:
            return f"""{self.kilolux} """
        
        if unit == IlluminanceUnits.Megalux:
            return f"""{self.megalux} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: IlluminanceUnits = IlluminanceUnits.Lux) -> string:
        """
        Get Illuminance unit abbreviation.
        Note! the default abbreviation for Illuminance is Lux.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == IlluminanceUnits.Lux:
            return """lx"""
        
        if unit_abbreviation == IlluminanceUnits.Millilux:
            return """"""
        
        if unit_abbreviation == IlluminanceUnits.Kilolux:
            return """"""
        
        if unit_abbreviation == IlluminanceUnits.Megalux:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Illuminance):
            raise TypeError("unsupported operand type(s) for +: 'Illuminance' and '{}'".format(type(other).__name__))
        return Illuminance(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Illuminance):
            raise TypeError("unsupported operand type(s) for *: 'Illuminance' and '{}'".format(type(other).__name__))
        return Illuminance(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Illuminance):
            raise TypeError("unsupported operand type(s) for -: 'Illuminance' and '{}'".format(type(other).__name__))
        return Illuminance(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Illuminance):
            raise TypeError("unsupported operand type(s) for /: 'Illuminance' and '{}'".format(type(other).__name__))
        return Illuminance(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Illuminance):
            raise TypeError("unsupported operand type(s) for %: 'Illuminance' and '{}'".format(type(other).__name__))
        return Illuminance(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Illuminance):
            raise TypeError("unsupported operand type(s) for **: 'Illuminance' and '{}'".format(type(other).__name__))
        return Illuminance(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Illuminance):
            raise TypeError("unsupported operand type(s) for ==: 'Illuminance' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Illuminance):
            raise TypeError("unsupported operand type(s) for <: 'Illuminance' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Illuminance):
            raise TypeError("unsupported operand type(s) for >: 'Illuminance' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Illuminance):
            raise TypeError("unsupported operand type(s) for <=: 'Illuminance' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Illuminance):
            raise TypeError("unsupported operand type(s) for >=: 'Illuminance' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class ElectricCurrentGradientUnits(Enum):
        """
            ElectricCurrentGradientUnits enumeration
        """
        
        AmperePerSecond = 'ampere_per_second'
        """
            
        """
        
        AmperePerMillisecond = 'ampere_per_millisecond'
        """
            
        """
        
        AmperePerMicrosecond = 'ampere_per_microsecond'
        """
            
        """
        
        AmperePerNanosecond = 'ampere_per_nanosecond'
        """
            
        """
        

class ElectricCurrentGradient:
    """
    In electromagnetism, the current gradient describes how the current changes in time.

    Args:
        value (float): The value.
        from_unit (ElectricCurrentGradientUnits): The ElectricCurrentGradient unit to create from, The default unit is AmperePerSecond
    """
    def __init__(self, value: float, from_unit: ElectricCurrentGradientUnits = ElectricCurrentGradientUnits.AmperePerSecond):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__amperes_per_second = None
        
        self.__amperes_per_millisecond = None
        
        self.__amperes_per_microsecond = None
        
        self.__amperes_per_nanosecond = None
        

    def __convert_from_base(self, from_unit: ElectricCurrentGradientUnits) -> float:
        value = self.__value
        
        if from_unit == ElectricCurrentGradientUnits.AmperePerSecond:
            return (value)
        
        if from_unit == ElectricCurrentGradientUnits.AmperePerMillisecond:
            return (value / 1e3)
        
        if from_unit == ElectricCurrentGradientUnits.AmperePerMicrosecond:
            return (value / 1e6)
        
        if from_unit == ElectricCurrentGradientUnits.AmperePerNanosecond:
            return (value / 1e9)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricCurrentGradientUnits) -> float:
        
        if to_unit == ElectricCurrentGradientUnits.AmperePerSecond:
            return (value)
        
        if to_unit == ElectricCurrentGradientUnits.AmperePerMillisecond:
            return (value * 1e3)
        
        if to_unit == ElectricCurrentGradientUnits.AmperePerMicrosecond:
            return (value * 1e6)
        
        if to_unit == ElectricCurrentGradientUnits.AmperePerNanosecond:
            return (value * 1e9)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_amperes_per_second(amperes_per_second: float):
        """
        Create a new instance of ElectricCurrentGradient from a value in amperes_per_second.

        

        :param meters: The ElectricCurrentGradient value in amperes_per_second.
        :type amperes_per_second: float
        :return: A new instance of ElectricCurrentGradient.
        :rtype: ElectricCurrentGradient
        """
        return ElectricCurrentGradient(amperes_per_second, ElectricCurrentGradientUnits.AmperePerSecond)

    
    @staticmethod
    def from_amperes_per_millisecond(amperes_per_millisecond: float):
        """
        Create a new instance of ElectricCurrentGradient from a value in amperes_per_millisecond.

        

        :param meters: The ElectricCurrentGradient value in amperes_per_millisecond.
        :type amperes_per_millisecond: float
        :return: A new instance of ElectricCurrentGradient.
        :rtype: ElectricCurrentGradient
        """
        return ElectricCurrentGradient(amperes_per_millisecond, ElectricCurrentGradientUnits.AmperePerMillisecond)

    
    @staticmethod
    def from_amperes_per_microsecond(amperes_per_microsecond: float):
        """
        Create a new instance of ElectricCurrentGradient from a value in amperes_per_microsecond.

        

        :param meters: The ElectricCurrentGradient value in amperes_per_microsecond.
        :type amperes_per_microsecond: float
        :return: A new instance of ElectricCurrentGradient.
        :rtype: ElectricCurrentGradient
        """
        return ElectricCurrentGradient(amperes_per_microsecond, ElectricCurrentGradientUnits.AmperePerMicrosecond)

    
    @staticmethod
    def from_amperes_per_nanosecond(amperes_per_nanosecond: float):
        """
        Create a new instance of ElectricCurrentGradient from a value in amperes_per_nanosecond.

        

        :param meters: The ElectricCurrentGradient value in amperes_per_nanosecond.
        :type amperes_per_nanosecond: float
        :return: A new instance of ElectricCurrentGradient.
        :rtype: ElectricCurrentGradient
        """
        return ElectricCurrentGradient(amperes_per_nanosecond, ElectricCurrentGradientUnits.AmperePerNanosecond)

    
    @property
    def amperes_per_second(self) -> float:
        """
        
        """
        if self.__amperes_per_second != None:
            return self.__amperes_per_second
        self.__amperes_per_second = self.__convert_from_base(ElectricCurrentGradientUnits.AmperePerSecond)
        return self.__amperes_per_second

    
    @property
    def amperes_per_millisecond(self) -> float:
        """
        
        """
        if self.__amperes_per_millisecond != None:
            return self.__amperes_per_millisecond
        self.__amperes_per_millisecond = self.__convert_from_base(ElectricCurrentGradientUnits.AmperePerMillisecond)
        return self.__amperes_per_millisecond

    
    @property
    def amperes_per_microsecond(self) -> float:
        """
        
        """
        if self.__amperes_per_microsecond != None:
            return self.__amperes_per_microsecond
        self.__amperes_per_microsecond = self.__convert_from_base(ElectricCurrentGradientUnits.AmperePerMicrosecond)
        return self.__amperes_per_microsecond

    
    @property
    def amperes_per_nanosecond(self) -> float:
        """
        
        """
        if self.__amperes_per_nanosecond != None:
            return self.__amperes_per_nanosecond
        self.__amperes_per_nanosecond = self.__convert_from_base(ElectricCurrentGradientUnits.AmperePerNanosecond)
        return self.__amperes_per_nanosecond

    
    def to_string(self, unit: ElectricCurrentGradientUnits = ElectricCurrentGradientUnits.AmperePerSecond) -> string:
        """
        Format the ElectricCurrentGradient to string.
        Note! the default format for ElectricCurrentGradient is AmperePerSecond.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == ElectricCurrentGradientUnits.AmperePerSecond:
            return f"""{self.amperes_per_second} A/s"""
        
        if unit == ElectricCurrentGradientUnits.AmperePerMillisecond:
            return f"""{self.amperes_per_millisecond} A/ms"""
        
        if unit == ElectricCurrentGradientUnits.AmperePerMicrosecond:
            return f"""{self.amperes_per_microsecond} A/μs"""
        
        if unit == ElectricCurrentGradientUnits.AmperePerNanosecond:
            return f"""{self.amperes_per_nanosecond} A/ns"""
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricCurrentGradientUnits = ElectricCurrentGradientUnits.AmperePerSecond) -> string:
        """
        Get ElectricCurrentGradient unit abbreviation.
        Note! the default abbreviation for ElectricCurrentGradient is AmperePerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricCurrentGradientUnits.AmperePerSecond:
            return """A/s"""
        
        if unit_abbreviation == ElectricCurrentGradientUnits.AmperePerMillisecond:
            return """A/ms"""
        
        if unit_abbreviation == ElectricCurrentGradientUnits.AmperePerMicrosecond:
            return """A/μs"""
        
        if unit_abbreviation == ElectricCurrentGradientUnits.AmperePerNanosecond:
            return """A/ns"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, ElectricCurrentGradient):
            raise TypeError("unsupported operand type(s) for +: 'ElectricCurrentGradient' and '{}'".format(type(other).__name__))
        return ElectricCurrentGradient(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, ElectricCurrentGradient):
            raise TypeError("unsupported operand type(s) for *: 'ElectricCurrentGradient' and '{}'".format(type(other).__name__))
        return ElectricCurrentGradient(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, ElectricCurrentGradient):
            raise TypeError("unsupported operand type(s) for -: 'ElectricCurrentGradient' and '{}'".format(type(other).__name__))
        return ElectricCurrentGradient(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, ElectricCurrentGradient):
            raise TypeError("unsupported operand type(s) for /: 'ElectricCurrentGradient' and '{}'".format(type(other).__name__))
        return ElectricCurrentGradient(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, ElectricCurrentGradient):
            raise TypeError("unsupported operand type(s) for %: 'ElectricCurrentGradient' and '{}'".format(type(other).__name__))
        return ElectricCurrentGradient(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, ElectricCurrentGradient):
            raise TypeError("unsupported operand type(s) for **: 'ElectricCurrentGradient' and '{}'".format(type(other).__name__))
        return ElectricCurrentGradient(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, ElectricCurrentGradient):
            raise TypeError("unsupported operand type(s) for ==: 'ElectricCurrentGradient' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, ElectricCurrentGradient):
            raise TypeError("unsupported operand type(s) for <: 'ElectricCurrentGradient' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, ElectricCurrentGradient):
            raise TypeError("unsupported operand type(s) for >: 'ElectricCurrentGradient' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, ElectricCurrentGradient):
            raise TypeError("unsupported operand type(s) for <=: 'ElectricCurrentGradient' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, ElectricCurrentGradient):
            raise TypeError("unsupported operand type(s) for >=: 'ElectricCurrentGradient' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value